﻿using Microsoft.Win32.SafeHandles;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using Syncfusion.DocIORenderer;
using System.Drawing;
using Syncfusion.Pdf;
using Syncfusion.Pdf.Barcode;
using Syncfusion.Pdf.Graphics;
using Syncfusion.Pdf.Parsing;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Threading.Tasks;
using System.Drawing.Imaging;

namespace ProfDev.Helpers {
    public class Exporter : IDisposable {
        private bool disposed = false;
        // Instantiate a SafeHandle instance.
        private SafeHandle handle = new SafeFileHandle(IntPtr.Zero, true);

        private MemoryStream _stream;
        private Dictionary<WParagraph, int> _pictureFields;

        public Stream MergeDataDuplicatePagesGet<T>(string templateFile, IEnumerable<T> data, string groupName, int numberOfPages, FormatType formatType = FormatType.Docx) where T : class {
            var docs = new List<WordDocument>();

            for (int i = 0; i < numberOfPages; i++) {
                docs.Add(GroupMergeDocData(templateFile, data, groupName, formatType));
            }

            var pdf = PDFDocCombine(docs);

            return pdf;
        }


        public Stream MergeDataPDFGet<T>(string templateFile, IEnumerable<T> data, string groupName, FormatType formatType = FormatType.Docx) where T : class {
            var doc = GroupMergeDocData(templateFile, data, groupName, formatType);

            var converter = new DocIORenderer();

            var pdf = converter.ConvertToPDF(doc);

            if (_stream == null) {
                _stream = new MemoryStream();
            }

            pdf.Save(_stream);
            pdf.Close();
            doc.Close();

            _stream.Position = 0;
            return _stream;
        }

        public void Dispose() {
            // Dispose of unmanaged resources.
            Dispose(true);
            // Suppress finalization.
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing) {
            if (disposed)
                return;

            if (disposing) {
                handle.Dispose();
                // Free any other managed objects here.
                //
            }

            disposed = true;
        }

        private static Image GenerateBarcodeImage(string barcodeText) {
            //Initialize a new PdfCode39Barcode instance.
            var barcode = new PdfQRBarcode();

            //barcode.XDimension = 3;

            barcode.Text = barcodeText;
            //Convert the barcode to image.
            Image barcodeImage = barcode.ToImage(new System.Drawing.SizeF(600, 600));
            return barcodeImage;
        }

        private WordDocument GroupMergeDocData<T>(string templateFile, IEnumerable<T> data, string groupName, FormatType formatType = FormatType.Docx) where T : class {
            WordDocument document;

            using (var fileStream = File.OpenRead(templateFile)) {
                document = new WordDocument(fileStream, formatType);
            }

            var mergeTable = new MailMergeDataTable(groupName, data);

            document.MailMerge.MergeField += new MergeFieldEventHandler(MergeField_InsertPageBreak);
            document.MailMerge.MergeImageField += new MergeImageFieldEventHandler(MergeImageField_InsertQRCode);

            _pictureFields = new Dictionary<WParagraph, int>();

            document.MailMerge.ExecuteNestedGroup(mergeTable);

            RemoveEmptyPage(ref document);

            foreach (var item in _pictureFields) {
                var paragraph = (WParagraph)item.Key;
                var picture = (WPicture)paragraph.Items[item.Value];

                if (picture != null) {
                    picture.Height = 75;
                    picture.Width = 75;
                    picture.TextWrappingStyle = TextWrappingStyle.Square;
                    picture.HorizontalAlignment = ShapeHorizontalAlignment.Right;
                    picture.VerticalPosition = 80;
                }
            }

            return document;
        }

        private void MergeImageField_InsertQRCode(object sender, MergeImageFieldEventArgs args) {
            if (args.FieldName == "SurveyURL") {
                //Generates barcode image for field value.
                Image barcodeImage = GenerateBarcodeImage(args.FieldValue.ToString());

                //Sets barcode image for merge field.
                var memoryStream = new MemoryStream();
                barcodeImage.Save(memoryStream, ImageFormat.Jpeg);
                memoryStream.Position = 0;

                args.ImageStream = memoryStream;

                if (_pictureFields != null) {
                    var paragraph = args.CurrentMergeField.OwnerParagraph;
                    _pictureFields.Add(paragraph, paragraph.Items.IndexOf(args.CurrentMergeField));
                }
            }
        }

        private static void MergeField_InsertPageBreak(object sender, MergeFieldEventArgs args) {
            if (args.FieldName == "StudentCount") {
                //Gets the owner paragraph 
                WParagraph paragraph = args.CurrentMergeField.OwnerParagraph;
                //Appends the page break 
                paragraph.AppendBreak(BreakType.PageBreak);
            }
        }

        private MemoryStream PDFDocCombine(List<WordDocument> docs) {
            var returnPDF = new PdfDocument();
            returnPDF.EnableMemoryOptimization = true;

            var converter = new DocIORenderer();


            foreach (var doc in docs) {
                var stream = new MemoryStream();
                var appendPDF = converter.ConvertToPDF(doc);
                appendPDF.Save(stream);
                returnPDF.Append(new PdfLoadedDocument(stream));
            }

            if (_stream == null) {
                _stream = new MemoryStream();
            }

            returnPDF.Save(_stream);
            returnPDF.Close();

            _stream.Position = 0;
            return _stream;
        }

        private void RemoveEmptyItems(WTextBody textBody) {
            //A flag to determine any renderable item found in the Word document.
            bool IsRenderableItem = false;
            //Iterates into textbody items.
            for (int itemIndex = textBody.ChildEntities.Count - 1; itemIndex >= 0 && !IsRenderableItem; itemIndex--) {
                #region Removes last empty paragraph
                //Checks item is empty paragraph and removes it.
                if (textBody.ChildEntities[itemIndex] is WParagraph) {
                    WParagraph paragraph = textBody.ChildEntities[itemIndex] as WParagraph;
                    //Iterates into paragraph
                    for (int pIndex = paragraph.Items.Count - 1; pIndex >= 0; pIndex--) {
                        ParagraphItem paragraphItem = paragraph.Items[pIndex];

                        //Removes page breaks
                        if ((paragraphItem is Break && (paragraphItem as Break).BreakType == BreakType.PageBreak))
                            paragraph.Items.RemoveAt(pIndex);

                        //Check paragraph contains any renderable items.
                        else if (!(paragraphItem is BookmarkStart || paragraphItem is BookmarkEnd)) {
                            //Found renderable item and break the iteration.
                            IsRenderableItem = true;
                            break;
                        }
                    }
                    //Remove empty paragraph and the paragraph with bookmarks only
                    if (paragraph.Items.Count == 0 || !IsRenderableItem)
                        textBody.ChildEntities.RemoveAt(itemIndex);
                }

                #endregion
            }
        }

        private void RemoveEmptyPage(ref WordDocument document) {
            WTextBody textBody;
            //Iterates sections in Word document.
            for (int i = document.Sections.Count - 1; i >= 0; i--) {
                //Accesses the Body of section where all the contents in document are apart
                textBody = document.Sections[i].Body;
                //Removes the last empty page in the Word document
                RemoveEmptyItems(textBody);
                //Removes the empty sections in the document
                if (textBody.ChildEntities.Count == 0) {
                    int SectionIndex = document.ChildEntities.IndexOf(document.Sections[i]);
                    document.ChildEntities.RemoveAt(SectionIndex);
                }
            }
        }
    }
}