﻿using mergeSample.Enums;
using mergeSample.Models;
using Microsoft.AspNetCore.Hosting;
using ProfDev.Helpers;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;

namespace mergeSample.Services {
    public class SignInSheetExportService : ISignInSheetExportService {
        private readonly IWebHostEnvironment _hostEnvironment;

        private DataContext _context { get; set; }

        public SignInSheetExportService(IWebHostEnvironment hostEnvironment) {
            _hostEnvironment = hostEnvironment;
            _context = new DataContext();
        }


        public Stream SignInSheetPDFGet(int district, int? workshopSectionID, DateTime? beginDate, DateTime? endDate, SignInSheetExportGroupBy groupBy, bool includeNoRegistration) {
            var attendees = _context.rptSignInSheet(district, workshopSectionID, null, beginDate, endDate, includeNoRegistration);
            var sessions = _context.WorkshopSectionSessionsGet(district, workshopSectionID, beginDate, endDate, includeNoRegistration);
            var presenters = _context.WorkshopSectionPresentersGet(district, workshopSectionID, beginDate, endDate, includeNoRegistration);

            List<SignInSheetMergeModel> mergeModel;

            if (groupBy == SignInSheetExportGroupBy.WorkshopSectionEmployee) {
                mergeModel = MergeModelBuild(sessions, presenters, attendees, includeNoRegistration);
            }
            else {
                mergeModel = MergeModelBuildByLocation(sessions, presenters, attendees, includeNoRegistration);
            }

            using (var exporter = new Exporter()) {
                var templateFile = MergeTemplateGet(district);

                return exporter.MergeDataPDFGet(templateFile, mergeModel.OrderBy(o => o.CourseNumber).ToList(), "Sections", Syncfusion.DocIO.FormatType.Docx);
            }
        }

        public Stream BlankSignInSheetsGet(int district, int numberOfPages, int? workshopSectionID, DateTime? beginDate, DateTime? endDate) {
            var sessions = _context.WorkshopSectionSessionsGet(district, workshopSectionID, beginDate, endDate, false);
            var presenters = _context.WorkshopSectionPresentersGet(district, workshopSectionID, beginDate, endDate, false);

            var attendees = new List<rptSignInSheetReturnModel>();
            for (int i = 0; i < 24; i++) {
                attendees.Add(new rptSignInSheetReturnModel() { FirstName = string.Empty, LastName = string.Empty, WorkLocation = string.Empty });
            }

            var mergeModel = MergeModelBuild(sessions, presenters, attendees, false);

            using (var exporter = new Exporter()) {
                var templateFile = MergeTemplateGet(district);

                return exporter.MergeDataDuplicatePagesGet(templateFile, mergeModel, "Sections", numberOfPages, Syncfusion.DocIO.FormatType.Docx);
            }
        }

        private List<SignInSheetMergeModel> MergeModelBuildByLocation(List<WorkshopSectionSessionsGetReturnModel> sessions, List<WorkshopSectionPresentersGetReturnModel> presenters, List<rptSignInSheetReturnModel> attendees, bool includeNoRegistration) {
            var locations = attendees.Select(s => s.WorkLocationName).Distinct().ToList();
            var mergeModel = new List<SignInSheetMergeModel>();

            foreach (var location in locations.OrderBy(o => o)) {
                var locationAttendees = attendees.Where(w => w.WorkLocationName == location).ToList();
                mergeModel.AddRange(MergeModelBuild(sessions, presenters, locationAttendees, includeNoRegistration));
            }

            return mergeModel;
        }

        private List<SignInSheetMergeModel> MergeModelBuild(List<WorkshopSectionSessionsGetReturnModel> sessions, List<WorkshopSectionPresentersGetReturnModel> presenters, List<rptSignInSheetReturnModel> attendees, bool includeNoRegistgration) {
            var returnList = new List<SignInSheetMergeModel>();
            foreach (var session in sessions) {
                var sessionAttendees = attendees.Where(w => w.WorkshopSectionSessionID == session.WorkshopSectionSessionID || w.WorkshopSectionSessionID == 0)
                                                .Select(s => new AttendeeModel() {
                                                    WorkshopSectionSessionID = s.WorkshopSectionSessionID,
                                                    WorkshopSectionID = s.WorkshopSectionID,
                                                    WorkLocation = s.WorkLocation,
                                                    WorkLocationName = s.WorkLocationName,
                                                    AttendanceHours = s.AttendanceHours,
                                                    CompensationTypeName = s.CompensationType,
                                                    District = s.District,
                                                    EmployeeID = s.EmployeeID,
                                                    FirstName = s.FirstName,
                                                    LastName = s.LastName,
                                                    RegistrationID = s.RegistrationID ?? 0,
                                                    SessionDate = s.SessionDate ?? DateTime.Now,
                                                    UserID = s.UserID ?? 0
                                                }).ToList();

                if (includeNoRegistgration || (!includeNoRegistgration && sessionAttendees.Any())) {
                    //TODO: is there a better way to do this?
                    for (int i = 0; i < sessionAttendees.Count; i++) {
                        sessionAttendees[i].Index = i + 1;
                    }

                    var newSession = new SignInSheetMergeModel() {
                        DistrictName = session.DistrictName,
                        WorkshopID = session.WorkshopID,
                        WorkshopName = session.WorkshopName,
                        WorkshopLocation = session.WorkshopLocation,
                        WorkshopLocationDescription = session.WorkshopLocationDescription,
                        WorkshopDate = (DateTime)session.WorkshopDate,
                        WorkshopBegin = (DateTime)session.WorkshopBegin,
                        WorkshopEnd = (DateTime)session.WorkshopEnd,
                        CourseNumber = (int)session.CourseNumber
                    };

                    if (session.SurveyRequirementID == (int)SurveyRequirements.EvaluationRequiredPerSession || session.SurveyRequirementID == (int)SurveyRequirements.EvaluationRequiredAfterWorkshop) {
                        newSession.SurveyURL = $"/Index?SurveyID={session.SurveyID}";
                    }

                    newSession.PresenterList = presenters.Where(w => w.WorkshopSectionSessionID == session.WorkshopSectionSessionID).Select(s => new PresenterModel() {
                        UserID = (int)s.UserID,
                        FirstName = s.NameFirst,
                        LastName = s.NameLast,
                        StaffID = s.StaffID,
                        WorkshopSectionID = (int)s.WorkshopSectionID,
                        WorkshopSectionStaffID = s.WorkshopSectionStaffID,
                        Title = s.Title,
                        IsActive = s.IsActive == 1
                    }).ToList();

                    newSession.Attendees = sessionAttendees;

                    returnList.Add(newSession);
                }
            }

            return returnList;
        }

        private string MergeTemplateGet(int district) {
            var districtConfig = _context.DistrictDefaults().Where(w => w.District == district).SingleOrDefault();

            var templateFile = $"{_hostEnvironment.WebRootPath}\\templates\\WorkshopSignInSheetQRCode.docx";

            if (districtConfig.IsEmployeeIDonSignInSheets) {
                templateFile = $"{_hostEnvironment.WebRootPath}\\templates\\WorkshopSignInSheetWithEmployeeIDQRCode.docx";
            }

            return templateFile;
        }
    }
}
